//
//  scViewController.m
//  sc
//
//  Created by xcc on 09/08/15.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "scViewController.h"
#import "testView.h"
#import "tileView.h"

@implementation scViewController

- (void)loadView {
	[super loadView];

	scrollview = [[TiledScrollView alloc] initWithFrame:self.view.bounds];
    [scrollview setDataSource:self];
    [[scrollview tileContainerView] setDelegate:self];
	[self.view addSubview:scrollview];
	//	タイル側内容部の最小最大スケール -2=1/4　〜　0=1/1 とする。
    [scrollview setMaximumResolution:6];
    [scrollview setMinimumResolution:-6];

	CGSize size = CGSizeMake(20000, 20000);
    [scrollview reloadDataWithNewContentSize:size];
    [scrollview setContentOffset:CGPointZero];

	//	画面の横幅いっぱいに全領域が表示されるスケールを計算し、設定する。ズーム範囲は 2^-10 〜 2^10
    [scrollview setMinimumZoomScale:pow(2, -10)];
    [scrollview setMaximumZoomScale:pow(2, 10)];
    float minScale = [scrollview frame].size.width  / size.width;
    [scrollview setZoomScale:minScale];    
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [scrollview release];
    [super dealloc];
}

/*
	シングルタップ対応
	なにもしない。
*/
-(void)view:(testView*)view gotSingleTapAtPoint:(CGPoint)tapLocation {
}

/*
	ダブルタップ対応
	tapLocationを中心にズームアウトする。
*/
-(void)view:(testView*)view gotDoubleTapAtPoint:(CGPoint)tapLocation {
    CGRect r;
	r.size.height = scrollview.frame.size.height * 0.5;
    r.size.width  = scrollview.frame.size.width  * 0.5;
	r.origin.x = tapLocation.x - (r.size.width  / 2.0);
    r.origin.y = tapLocation.y - (r.size.height / 2.0);
    [scrollview zoomToRect:r animated:YES];
}

/*
	2フィンガータップ対応
	tapLocationを中心に1：1に戻す。
*/
- (void)view:(testView *)view gotTwoFingerTapAtPoint:(CGPoint)tapLocation {
    CGRect r;
	r.size.height = scrollview.frame.size.height;
    r.size.width  = scrollview.frame.size.width;
	r.origin.x = tapLocation.x - (r.size.width  / 2.0);
    r.origin.y = tapLocation.y - (r.size.height / 2.0);
    [scrollview zoomToRect:r animated:YES];
}

/*
	指定された縦横インディックスに利用するUIViewを返す。
*/
- (UIView *)tiledScrollView:(TiledScrollView *)tiledScrollView tileForRow:(int)row column:(int)column resolution:(int)resolution {
	//	まず、再利用可能なUIImageViewがあるか確認。
	tileView *tile = (tileView *)[scrollview dequeueReusableTile];
	if (!tile) {
		//	frameはTiledScrollView側が調整するので気にせずCGRectZeroを指定しておく。
		tile = [[[tileView alloc] initWithFrame:CGRectZero] autorelease]; 
		//	タイルの右や、下は、内容部が無い場合がある。この時にUIViewが内容部をスケーリングしてしまうのを止める。
		[tile setContentMode:UIViewContentModeTopLeft];
	}
	double scale = pow(2, resolution * -1);
	tile.logical_origin = CGPointMake(column * tiledScrollView.tileSize.width * scale, row * tiledScrollView.tileSize.height * scale);
	tile.logical_scale = pow(2, resolution);
	[tile setNeedsDisplay];
    return tile;
}

@end
