//
//  testView.m
//  sc
//
//  Created by xcc on 09/08/15.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "testView.h"


@implementation testView
@synthesize delegate;
@synthesize drawSelf;

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        [self setMultipleTouchEnabled:YES];
		twoFingerTapIsPossible = YES;
		multipleTouches = NO;		
    }
    return self;
}

- (void)drawRect:(CGRect)rect {
	if (drawSelf == NO)
		return;
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSetRGBFillColor(context, 0.5, 0.5, 0.5, 1.0);
	CGContextSetRGBStrokeColor(context, 0.5, 0.5, 0.5, 1.0);
	UIFont *font = [UIFont systemFontOfSize:12]; 
	CGPoint origin = self.frame.origin;
	double left = self.bounds.origin.x;
	double top = self.bounds.origin.y;
	double right = left + self.bounds.size.width;
	double bottom = top + self.bounds.size.height;
	for (double v = top; v < bottom; v += 100.0) {
		for (double h = left; h < right; h += 100.0) {
			NSString* str = [NSString stringWithFormat:@"[%.0lf, %.0lf]", h + origin.x, v + origin.y];
			[str drawAtPoint:CGPointMake(h, v) withFont:font]; 
		}
	}
	for (double v = top; v < bottom; v += 100.0) {
		CGContextMoveToPoint(context, left, v);
		CGContextAddLineToPoint(context, right, v);
		CGContextStrokePath(context);	
	}
	for (double h = left; h < right; h += 100.0) {
		CGContextMoveToPoint(context, h, top);
		CGContextAddLineToPoint(context, h, bottom);
		CGContextStrokePath(context);	
	}
}

- (void)dealloc {
    [super dealloc];
}


- (void)setZoomScale:(double)inScale {
}

/*
	シングルタップ
*/
- (void)handleSingleTap {
    if ([delegate respondsToSelector:@selector(view:gotSingleTapAtPoint:)])
        [delegate view:self gotSingleTapAtPoint:tapLocation];
}

/*
	ダブルタップ
*/
- (void)handleDoubleTap {
    if ([delegate respondsToSelector:@selector(view:gotDoubleTapAtPoint:)])
        [delegate view:self gotDoubleTapAtPoint:tapLocation];
}

- (void)handleTwoFingerTap {
    if ([delegate respondsToSelector:@selector(view:gotTwoFingerTapAtPoint:)])
        [delegate view:self gotTwoFingerTapAtPoint:tapLocation];
}

#define DOUBLE_TAP_DELAY 0.35

/*
	指が触れた。
	こちらはUIScrollViewに埋め込まれていてもいなくても、同じタイミングで呼び出される。
*/
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	/*
		もし、このメソッドの前にhandleSingleTapが登録されていて実行待ちであれば
		DOUBLE_TAP_DELAY間隔のあいだにtouchesBeganが2度来た事になる。
		それはダブルタップとみなすべきなので、handleSingleTap登録は解除する。
		もともと登録が無ければ、この処理はなにも影響をおよぼさない。
	*/
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(handleSingleTap) object:nil];
	/*
		eventから、現在追跡中の自分に対するタッチの数を得る事ができる。
		touchesの方からはtouchesBeganを引き起こした指の数だけしか得られない事に注意。
	*/
	if ([[event touchesForView:self] count] > 1)
        multipleTouches = YES;
	/*
		同じく、追跡中の自分に対するタッチの数が2より大きければ2フィンガータップとはならない。
	*/
    if ([[event touchesForView:self] count] > 2)
        twoFingerTapIsPossible = NO;
	printf("testView touchesBegan twoFingerTapIsPossible = %s multipleTouches = %s\n",
		twoFingerTapIsPossible?"YES":"NO",
		multipleTouches?"YES":"NO");
 	printf("   [touches count] = %d [[event touchesForView:self] count] = %d\n",[touches count], [[event touchesForView:self] count]);
}

static CGPoint midpointBetweenPoints(CGPoint a, CGPoint b) {
    CGFloat x = (a.x + b.x) / 2.0;
    CGFloat y = (a.y + b.y) / 2.0;
    return CGPointMake(x, y);
}

/*
	指が離れた。
	UIScrollViewに埋め込まれていると、予想外の順序で呼び出される。
	ここでは、UIScrollViewに埋め込まれていない場合のシーケンスで対応している。
*/
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	/*
		eventから、現在追跡中の自分に対するタッチの数を得る事ができる。この数とtouchesの数を比較する事で
		最後の指が離れたかどうかを判定できる。
		例えば2本の指が触れている状態から1本だけ指が離れた場合、
			[touches count] = 1
			[[event touchesForView:self] count] = 2
		で、すべての指が離れたとは言えない。
	*/
    BOOL allTouchesEnded = ([touches count] == [[event touchesForView:self] count]);   
    if (!multipleTouches) {
		//	まず、現在追跡中の自分に対するタッチの数が 1 である場合（シングルフィンガー）
        UITouch *touch = [touches anyObject];
        tapLocation = [touch locationInView:self];
		//	タップの数で、シングルタップかダブルタップか判断して実行。
        if ([touch tapCount] == 1) {
            [self performSelector:@selector(handleSingleTap) withObject:nil afterDelay:DOUBLE_TAP_DELAY];
        } else if([touch tapCount] == 2) {
            [self handleDoubleTap];
        }
    } else if (multipleTouches && twoFingerTapIsPossible) { 
		//	2本指でのジェスチャー
        if ([touches count] == 2 && allTouchesEnded) {
			//	2本指でのジェスチャーで2本とも離れた。まず各指のタップ数を求める。
            int i = 0; 
            int tapCounts[2];
			CGPoint tapLocations[2];
            for (UITouch *touch in touches) {
                tapCounts[i]    = [touch tapCount];
                tapLocations[i] = [touch locationInView:self];
                i++;
            }
            if ((tapCounts[0] == 1) && (tapCounts[1] == 1)) {
				//	両方の指ともシングルタップ
                tapLocation = midpointBetweenPoints(tapLocations[0], tapLocations[1]);
                [self handleTwoFingerTap];
            }
        } else if ([touches count] == 1 && !allTouchesEnded) {
			//	あと1本、指が残っている。
            UITouch *touch = [touches anyObject];
            if ([touch tapCount] == 1) {
                //	注意1）残りの指が離れた時の準備
                tapLocation = [touch locationInView:self];
            } else {
				//	何回もタップされているので、もはや2フィンガータップにはならない。
                twoFingerTapIsPossible = NO;
            }
        } else if ([touches count] == 1 && allTouchesEnded) {
			//	複数の指がタッチしていたが、順に離れていき、最後の指が離れた。
            UITouch *touch = [touches anyObject];
            if ([touch tapCount] == 1) {
				//	注意1）で保存しておいた最初に離れた方の指の座標を使って、最終的なtapLocationを決定。
                tapLocation = midpointBetweenPoints(tapLocation, [touch locationInView:self]);
                [self handleTwoFingerTap];
            }
        }
    }
        
    if (allTouchesEnded) {
		//	すべての指が離れたのでリセット
        twoFingerTapIsPossible = YES;
        multipleTouches = NO;
    }
	printf("testView touchesEnded twoFingerTapIsPossible = %s multipleTouches = %s\n",
		twoFingerTapIsPossible?"YES":"NO",
		multipleTouches?"YES":"NO");
 	printf("   [touches count] = %d [[event touchesForView:self] count] = %d\n",[touches count], [[event touchesForView:self] count]);
}

/*
	電話がかかったときではなく、UIScrollViewがフリックやピンチを確認した時も送られてくる。
*/
- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
	//	すべての指が離れたのでリセット
    twoFingerTapIsPossible = YES;
    multipleTouches = NO;
	printf("testView touchesCancelled twoFingerTapIsPossible = %s multipleTouches = %s\n",
		twoFingerTapIsPossible?"YES":"NO",
		multipleTouches?"YES":"NO");
 	printf("   [touches count] = %d [[event touchesForView:self] count] = %d\n",[touches count], [[event touchesForView:self] count]);
}

@end
