//
//  listViewController.m
//  list
//
//  Created by kunii on 09/08/26.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "listViewController.h"

@implementation listViewController

#define THUMB_WIDTH 100		//	サムネイルビューの幅
#define THUMB_HEIGHT 150	//	〃　高さ
#define MARGIN 10			//	サムネイルビュー間のすきま

#define AUTOSCROLL_THRESHOLD 30

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

static UIScrollView* createThumbScrollView(CGRect inFrame)
{    
	UIScrollView* thumbScrollView = [[UIScrollView alloc] initWithFrame:inFrame];
	thumbScrollView.canCancelContentTouches = NO;
//	thumbScrollView.delaysContentTouches = NO;
	[thumbScrollView setClipsToBounds:NO];
	return thumbScrollView;
}

// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
	[super loadView];
	//	スクロールビュー作成
	float scrollViewHeight = MARGIN + THUMB_HEIGHT + MARGIN;
	float scrollViewWidth  = [[self view] bounds].size.width;
	thumbScrollView = createThumbScrollView(CGRectMake(0, 0, scrollViewWidth, scrollViewHeight));
	//	スクロールビューにサムネイルビューを8枚埋め込む
	CGRect frame = CGRectMake(MARGIN, MARGIN, THUMB_WIDTH, THUMB_HEIGHT);	//	THUMB_WIDTH x THUMB_HEIGHT をサムネイルビューとする
	for (int i = 0; i < 8; i++) {
		thumbView *view = [[thumbView alloc] initWithFrame:frame];
		view.delegate = self;
		view.backgroundColor = [UIColor greenColor];
		[thumbScrollView addSubview:view];
		[view release];
		frame.origin.x += (frame.size.width + MARGIN);			//	横にずらす
	}
	[thumbScrollView setContentSize:CGSizeMake(frame.origin.x, scrollViewHeight)];

	//	スクロールビューを埋め込む入れ物を作成
	CGRect bounds = [[self view] bounds];
	frame = CGRectMake(CGRectGetMinX(bounds), CGRectGetMaxY(bounds) - [thumbScrollView frame].size.height, bounds.size.width, [thumbScrollView frame].size.height);
	slideUpView = [[UIView alloc] initWithFrame:frame];
	//	暗い半透明の黒の背景とする
	[slideUpView setBackgroundColor:[UIColor blackColor]];
	[slideUpView setOpaque:NO];
	[slideUpView setAlpha:0.75];
	[[self view] addSubview:slideUpView];
	//	スクロールビュー埋めこみ
	[slideUpView addSubview:thumbScrollView];
}


/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

/*
	タッチ位置と両端との距離（proximity）が少ないほど大きくスクロールするようにスクロール幅を計算する。
*/
- (float)autoscrollDistanceForProximityToEdge:(float)proximity {
    return ceilf((AUTOSCROLL_THRESHOLD - proximity) / 5.0);
}

/*
	autoscrollDistanceの値を現在のスクロール位置に加算する事で、thumbScrollViewのframe外が表示されないようにautoscrollDistanceを調整。
*/
- (void)legalizeAutoscrollDistance {
    float minimumLegalDistance = [thumbScrollView contentOffset].x * -1;
    float maximumLegalDistance = [thumbScrollView contentSize].width - ([thumbScrollView frame].size.width + [thumbScrollView contentOffset].x);
    autoscrollDistance = MAX(autoscrollDistance, minimumLegalDistance);
    autoscrollDistance = MIN(autoscrollDistance, maximumLegalDistance);
}

/*
	1/60ごとにautoscrollDistance分のスクロール処理をおこなう。
*/
- (void)autoscrollTimerFired:(NSTimer*)timer {
    [self legalizeAutoscrollDistance];
	CGPoint contentOffset = [thumbScrollView contentOffset];	//	現在の表示オフセットを求める。
    contentOffset.x += autoscrollDistance;
    [thumbScrollView setContentOffset:contentOffset];			//	新しい表示オフセットを設定することでスクロール発生。    
	//	スクロールするとドラッグ中のthumbViewも一緒に動く。現在の位置にとどまるように逆の加算をおこなう。
    thumbView *thumb = (thumbView *)[timer userInfo];
    [thumb moveByOffset:CGPointMake(autoscrollDistance, 0)];
}

/*
	オートスクロールが必要であればオートスクロール処理をタイマー割り込みでおこなうように設定する。
*/
- (void)maybeAutoscrollForThumb:(thumbView *)thumb {
    autoscrollDistance = 0;
    if (CGRectIntersectsRect([thumb frame], [thumbScrollView bounds])) {
		//	thumbのframeはthumbScrollViewのboundsと接触（内部にある場合も含む）している。
        CGPoint touchLocation = [thumb convertPoint:[thumb touchLocation] toView:thumbScrollView];	//	thumbScrollView上でのthumbのドラッグ開始位置を計算
		//	thumbのドラッグ開始位置と両端との距離をそれぞれ求める。
        float distanceFromLeftEdge  = touchLocation.x - CGRectGetMinX([thumbScrollView bounds]);
        float distanceFromRightEdge = CGRectGetMaxX([thumbScrollView bounds]) - touchLocation.x;
        if (distanceFromLeftEdge < AUTOSCROLL_THRESHOLD) {
			//	タッチ位置が左から右にスクロールを必要とする位置。
            autoscrollDistance = [self autoscrollDistanceForProximityToEdge:distanceFromLeftEdge] * -1;	//	負の方向なので -1 をかける。
        } else if (distanceFromRightEdge < AUTOSCROLL_THRESHOLD) {
			//	タッチ位置が右から左にスクロールを必要とする位置。
            autoscrollDistance = [self autoscrollDistanceForProximityToEdge:distanceFromRightEdge];
        }        
    }
    if (autoscrollDistance == 0) {
		//	0ということはスクロールが必要ではない。
        [autoscrollTimer invalidate];			//	タイマーを止める。
        autoscrollTimer = nil;
    } else if (autoscrollTimer == nil) {
		//	そうでなければオートスクロールさせる。ただし、autoscrollTimerがnilでないなら、すでにオートスクロール中なのでなにもしなくていい。
        autoscrollTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0 / 60.0)
                                                           target:self 
                                                         selector:@selector(autoscrollTimerFired:) 
                                                         userInfo:thumb 
                                                          repeats:YES];
    } 
}

/*
	thumbViewをアニメーション付きで再配置する。
	再配置するときは、再配置する位置以降にあるthumbViewを前詰め、または後詰めして間をあける。
	この機能はthumbScrollView内にある時だけ動作する。
*/
-(void)reorderingTumbs:(thumbView*)draggingThumb {
    if (CGRectIntersectsRect([draggingThumb frame], [thumbScrollView bounds])) {//	thumbScrollView内にある時だけ機能させる
		//	ドラッグ中のthumbViewが元の場所より右にいるならYES
        BOOL draggingRight = [draggingThumb frame].origin.x > [draggingThumb home].origin.x ? YES : NO;
		//	移動するべきthumbViewの配列の準備
        NSMutableArray *thumbsToShift = [[NSMutableArray alloc] init];
		//	thumbScrollView上でのタッチ位置を得る。
        CGPoint touchLocation = [draggingThumb convertPoint:[draggingThumb touchLocation] toView:thumbScrollView];
		//	ドラッグ中のthumbViewを再配置するために、移動させられる他のthumbViewが存在する範囲をもとめる。
        float minX = draggingRight ? CGRectGetMaxX([draggingThumb home]) : touchLocation.x;
        float maxX = draggingRight ? touchLocation.x : CGRectGetMinX([draggingThumb home]);
		//	全thumbViewに対し、再配置するために移動させられるかどうかを判断する。
        for (thumbView *thumb in [thumbScrollView subviews]) {
            if (thumb == draggingThumb)								//	ドラッグ中のthumbViewなので対象外
				continue;
            if (![thumb isMemberOfClass:[thumbView class]])			//	このUIViewインスタンスはthumbViewではない
				continue;
            float thumbMidpoint = CGRectGetMidX([thumb home]);		//	thumbViewの中心点を求める
            if (thumbMidpoint >= minX && thumbMidpoint <= maxX) {	//	移動対象
                [thumbsToShift addObject:thumb];
            }
        }
		//	再配置するための移動方向と移動量を決定する。ドラッグ中のthumbViewが以前の位置より右に再配置されるなら、他のthumbViewは左に移動することになる。
        float otherThumbShift = ([draggingThumb home].size.width + MARGIN) * (draggingRight ? -1 : 1);
		//	ドラッグ中のthumbViewの新しい初期位置（home）に移動するためのずれ量
        float draggingThumbShift = 0.0;
		//	ドラッグ中のthumbView以外を再配置
        for (thumbView *otherThumb in thumbsToShift) {
            CGRect home = [otherThumb home];
            home.origin.x += otherThumbShift;
            [otherThumb setHome:home];
            [otherThumb goHome];									//	ここで移動アニメーションが発生する。
            draggingThumbShift += ([otherThumb frame].size.width + MARGIN) * (draggingRight ? 1 : -1);
        }
		//	ドラッグ中のthumbViewの初期位置を設定、ただし、まだ移動中なのでgoHomeは呼ばない。
        CGRect home = [draggingThumb home];
        home.origin.x += draggingThumbShift;
        [draggingThumb setHome:home];
    }
}

- (void)thumbViewStartedTracking:(thumbView *)tiv {
    [thumbScrollView bringSubviewToFront:tiv];			//	一番手前にもってくる。
}

- (void)thumbViewMoved:(thumbView *)draggingThumb {
    [self maybeAutoscrollForThumb:draggingThumb];		//	オートスクロールが必要であればおこなう。
	[self reorderingTumbs:draggingThumb];				//	必要なら再配置をおこなう。

}

- (void)thumbViewStoppedTracking:(thumbView *)tiv {
    [autoscrollTimer invalidate];						//	ドラッグが終わったので、オートスクロールもただちに停止。
    autoscrollTimer = nil;
}


@end
